#!/bin/bash
#

# build_dk2nu.sh 

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e9:r6|e9:r5|e7:r5> <debug|opt|prof> [tar]"
   echo "        Note that only the e9 build can be used with root 6 (e9:r6)"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    PRODUCTS=${product_dir}:${PRODUCTS}
    
    setup ssibuildshims ${ssibuildshims_version}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=dk2nu
origpkgver=v01_03_00
pkgver=${origpkgver}a
cmake_version=v3_3_2
ssibuildshims_version=v0_09_00
pkgtarname=${package}-${origpkgver}

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------


case ${extraqual} in
debug) extra_command="-DCMAKE_BUILD_TYPE=Debug";;
prof)  extra_command="-DCMAKE_BUILD_TYPE=MinSizeRel";;
opt)   extra_command="-DCMAKE_BUILD_TYPE=Release";;
* )
   echo "ERROR: please specify debug, opt, or prof"
   usage
   exit 1
;;
esac

if [[ "${basequal}" == e[79]:r5 ]]
then
  cxxflg="-std=c++14"
  extra_command="${extra_command} -DCMAKE_CXX_FLAGS=${cxxflg}"
elif [[ "${basequal}" == e9:r6 ]]
then
  cxxflg="-std=c++14"
  extra_command="${extra_command} -DCMAKE_CXX_FLAGS=${cxxflg}"
elif [ "${basequal}" = "i1" ]
then
  cxxflg="-std=c++11"
  extra_command="${extra_command} -DCMAKE_CXX_FLAGS=${cxxflg}"
else
  die "Qualifier $basequal not recognized."
fi

echo "Building ${package} in ${blddir}"

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || die "ERROR: fake setup failed"

# We need cmake
setup -B cmake ${cmake_version} || die "ERROR: setup -B cmake ${cmake_version} failed"

if [ -z ${DK2NU} ]
then
   echo "ERROR: failed to setup dk2nu"
   exit 1
else
   echo "DK2NU: ${DK2NU}"
fi

if [ -z ${GENIE} ]
then
   echo "ERROR: failed to setup genie"
   exit 1
else
   echo "GENIE: ${GENIE}"
fi

if [ -z ${ROOTSYS} ]
then
   echo "ERROR: failed to setup root"
   exit 1
else
   echo "ROOTSYS: ${ROOTSYS}"
fi

if [ -z ${LIBXML2_INC} ]
then
   echo "ERROR: failed to setup libxml"
   exit 1
else
   echo "LIBXML2_INC: ${LIBXML2_INC}"
fi

if [ -z ${LOG4CPP_INC} ]
then
   echo "ERROR: failed to setup log4cpp"
   exit 1
else
   echo "LOG4CPP_INC: ${LOG4CPP_INC}"
fi

set -x

cd ${blddir}

env CC=gcc CXX=g++ FC=gfortran cmake \
    -DCMAKE_INSTALL_PREFIX=${pkgdir} ${extra_command} ${sourcedir}/dk2nu || \
  die "ERROR: cmake step failed"
make VERBOSE=t all install || die "ERROR: make failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir} || die "ERROR: setup failed"
echo "${package} is installed at ${DK2NU}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
